function [Stress,Strain]=pCDMstress(X,Y,Z,X0,Y0,depth,omegaX,omegaY,...
    omegaZ,DVx,DVy,DVz,mu,lambda)
% pCDMstress
% calculates the strains and stresses associated with a point CDM in a 
% uniform elastic half-space.
% 
% pCDM: point Compound Dislocation Model
% PTD: Point Tensile Dislocation
% EFCS: Earth-Fixed Coordinate System
%
% INPUTS
% X, Y and Z:
% Coordinates of calculation points in EFCS (East, North, Up). X, Y and Z
% must have the same size.
%
% X0, Y0 and depth:
% Horizontal coordinates (in EFCS) and depth of the point CDM. The depth
% must be a positive value. X0, Y0 and depth have the same unit as X, Y and
% Z.
%
% omegaX, omegaY and omegaZ:
% Clockwise rotation angles about X, Y and Z axes, respectively, that 
% specify the orientation of the point CDM in space. The input values must 
% be in degrees.
%
% DVx, DVy and DVz:
% Potencies of the PTDs that before applying the rotations are normal to 
% the X, Y and Z axes, respectively. The potency has the unit of volume 
% (the unit of displacements and CDM semi-axes to the power of 3).
%
% mu and lambda:
% Lam constants.
% 
% 
% OUTPUTS
% Stress:
% Calculated stress tensor components in EFCS. The six columns of Stress
% are Sxx, Syy, Szz, Sxy, Sxz and Syz, respectively. The stress components
% have the same unit as Lam constants.
%
% Strain:
% Calculated strain tensor components in EFCS. The six columns of Strain
% are Exx, Eyy, Ezz, Exy, Exz and Eyz, respectively. The strain components
% are dimensionless.
%
%
% Example: Calculate and plot the first component of stress tensor on a 
% regular grid below the surface
%
% [X,Y,Z] = meshgrid(-7:.02:7,-5:.02:5,-1);
% X0 = 0.5; Y0 = -0.25; depth = 2.75; omegaX = 5; omegaY = -8; omegaZ = 30;
% DVx = 0.00144; DVy = 0.00128; DVz = 0.00072; mu = 2e9; lambda = 2e9;
% [Stress,Strain] = pCDMstress(X,Y,Z,X0,Y0,depth,omegaX,omegaY,omegaZ,...
%     DVx,DVy,DVz,mu,lambda);
% figure
% surf(X,Y,reshape(Stress(:,1),size(X)),'edgecolor','none')
% view(2)
% axis equal
% axis tight
% set(gcf,'renderer','painters')

% Reference journal articles:
% 1)
% Nikkhoo, M., Walter, T. R., Lundgren, P. R., Prats-Iraola, P. (2017):
% Compound dislocation models (CDMs) for volcano deformation analyses.
% Submitted to Geophysical Journal International, doi: 10.1093/gji/ggw427
% 
% 2)
% Okada, Y. (1992):
% Internal deformation due to shear and tensile faults in a half-space, 
% Bull. seism. Soc. Am., 82(2), 10181040.

% Copyright (c) 2016 Mehdi Nikkhoo
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files
% (the "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.

% I appreciate any comments or bug reports.

% Mehdi Nikkhoo
% Created: 2016.7.4
% Last modified: 2022.10.24
%
% Section 2.1, Physics of Earthquakes and Volcanoes
% Department 2, Geophysics
% Helmholtz Centre Potsdam
% German Research Centre for Geosciences (GFZ)
%
% email:
% mehdi.nikkhoo@gfz-potsdam.de
% mehdi.nikkhoo@gmail.com
%
% website:
% http://www.volcanodeformation.com

% The potencies must have the same sign
DVsign = [sign(DVx) sign(DVy) sign(DVz)];
if any(DVsign>0) && any(DVsign<0)
    error('Input error: DVx, DVy and DVz must have the same sign!')
end

X = X(:);
Y = Y(:);
Z = Z(:);

Rx = [1 0 0;0 cosd(omegaX) sind(omegaX);0 -sind(omegaX) cosd(omegaX)];
Ry = [cosd(omegaY) 0 -sind(omegaY);0 1 0;sind(omegaY) 0 cosd(omegaY)];
Rz = [cosd(omegaZ) sind(omegaZ) 0;-sind(omegaZ) cosd(omegaZ) 0;0 0 1];
R = Rz*Ry*Rx;

Vstrike1 = [-R(2,1),R(1,1),0];
Vstrike1 = Vstrike1/norm(Vstrike1);
strike1 = atan2(Vstrike1(1),Vstrike1(2))*180/pi;
if isnan(strike1)
    strike1 = 0;
end
dip1 = acosd(R(3,1));

Vstrike2 = [-R(2,2),R(1,2),0];
Vstrike2 = Vstrike2/norm(Vstrike2);
strike2 = atan2(Vstrike2(1),Vstrike2(2))*180/pi;
if isnan(strike2)
    strike2 = 0;
end
dip2 = acosd(R(3,2));

Vstrike3 = [-R(2,3),R(1,3),0];
Vstrike3 = Vstrike3/norm(Vstrike3);
strike3 = atan2(Vstrike3(1),Vstrike3(2))*180/pi;
if isnan(strike3)
    strike3 = 0;
end
dip3 = acosd(R(3,3));

% Calculate contribution of the first PTD
if DVx~=0
    [Sts1,Str1] = PTDstressHS(X,Y,Z,X0,Y0,depth,strike1,dip1,DVx,mu,...
        lambda);
else
    Sts1 = zeros(numel(X),6);
    Str1 = zeros(numel(X),6);
end

% Calculate contribution of the second PTD
if DVy~=0
    [Sts2,Str2] = PTDstressHS(X,Y,Z,X0,Y0,depth,strike2,dip2,DVy,mu,...
        lambda);
else
    Sts2 = zeros(numel(X),6);
    Str2 = zeros(numel(X),6);
end

% Calculate contribution of the third PTD
if DVz~=0
    [Sts3,Str3] = PTDstressHS(X,Y,Z,X0,Y0,depth,strike3,dip3,DVz,mu,...
        lambda);
else
    Sts3 = zeros(numel(X),6);
    Str3 = zeros(numel(X),6);
end

Stress = Sts1+Sts2+Sts3;
Strain = Str1+Str2+Str3;


function [Stress,Strain]=PTDstressHS(X,Y,Z,X0,Y0,depth,strike,dip,DV,mu,...
    lambda)
% PTDstressHS calculates the internal stresses due to a point tensile 
% dislocation (PTD) in an elastic half-space (Okada, 1992).

sf = 1/2/pi; 
DVs = DV*sf;

x = X(:)-X0;
y = Y(:)-Y0;
z = Z(:);

beta = strike-90;
Rz = [cosd(beta) -sind(beta) 0;sind(beta) cosd(beta) 0;0 0 1];
r_beta = Rz*[x y z]';
x = r_beta(1,:)';
y = r_beta(2,:)';

dp = depth-z;
dn = depth+z;
rp = (x.^2+y.^2+dp.^2).^0.5;
rn = (x.^2+y.^2+dn.^2).^0.5;

alpha = (lambda+mu)/(lambda+2*mu);

pp = y*cosd(dip)+dp*sind(dip);
pn = y*cosd(dip)+dn*sind(dip);
qp = y*sind(dip)-dp*cosd(dip);
qn = y*sind(dip)-dn*cosd(dip);
sp = pp*sind(dip)+qp*cosd(dip);
sn = pn*sind(dip)+qn*cosd(dip);
tp = pp*cosd(dip)-qp*sind(dip);
tn = pn*cosd(dip)-qn*sind(dip);
Up = sind(dip)-5*y.*qp./rp.^2;
Un = sind(dip)-5*y.*qn./rn.^2;
Wp = sind(dip)+Up;
Wn = sind(dip)+Un;
Uip = cosd(dip)+5*dp.*qp./rp.^2;
Uin = cosd(dip)+5*dn.*qn./rn.^2;
Wip = cosd(dip)+Uip;
Win = cosd(dip)+Uin;

% ru_rx
[rue_rxAp,run_rxAp,ruv_rxAp] = ruA_rx(x,y,dp,rp,qp,sp,tp,alpha);
[rue_rxAn,run_rxAn,ruv_rxAn] = ruA_rx(x,y,dn,rn,qn,sn,tn,alpha);
[rue_rxB,run_rxB,ruv_rxB] = ruB_rx(x,y,dp,depth,dip,rp,qp,alpha);
[rue_rxC,run_rxC,ruv_rxC] = ruC_rx(x,y,z,dp,depth,dip,rp,qp,sp,tp,alpha);

rue_rx = DVs*(rue_rxAp-rue_rxAn+rue_rxB+z.*rue_rxC);
run_rx = DVs*(run_rxAp-run_rxAn+run_rxB+z.*run_rxC);
ruv_rx = DVs*(ruv_rxAp-ruv_rxAn+ruv_rxB+z.*ruv_rxC);

% ru_ry
[rue_ryAp,run_ryAp,ruv_ryAp] = ruA_ry(x,y,dp,dip,rp,qp,sp,tp,alpha,Wp);
[rue_ryAn,run_ryAn,ruv_ryAn] = ruA_ry(x,y,dn,dip,rn,qn,sn,tn,alpha,Wn);
[rue_ryB,run_ryB,ruv_ryB] = ruB_ry(x,y,dp,depth,dip,rp,qp,alpha,Wp);
[rue_ryC,run_ryC,ruv_ryC] = ruC_ry(x,y,z,dp,depth,dip,rp,qp,sp,tp,alpha);

rue_ry = DVs*(rue_ryAp-rue_ryAn+rue_ryB+z.*rue_ryC);
run_ry = DVs*(run_ryAp-run_ryAn+run_ryB+z.*run_ryC);
ruv_ry = DVs*(ruv_ryAp-ruv_ryAn+ruv_ryB+z.*ruv_ryC);

% ru_rz
[rue_rzAp,run_rzAp,ruv_rzAp] = ruA_rz(x,y,dp,dip,rp,qp,sp,tp,alpha,Wip);
[rue_rzAn,run_rzAn,ruv_rzAn] = ruA_rz(x,y,dn,dip,rn,qn,sn,tn,alpha,Win);
[rue_rzB,run_rzB,ruv_rzB] = ruB_rz(x,y,dp,depth,dip,rp,qp,alpha,Wip);
[rue_rzC,run_rzC,ruv_rzC] = ruC_rz(x,y,z,dp,depth,dip,rp,qp,sp,alpha);
[ue_C,un_C,uv_C] = uC(x,y,z,dp,depth,dip,rp,qp,sp,tp,alpha);

rue_rz = DVs*(rue_rzAp+rue_rzAn+rue_rzB+ue_C+z.*rue_rzC);
run_rz = DVs*(run_rzAp+run_rzAn+run_rzB+un_C+z.*run_rzC);
ruv_rz = DVs*(ruv_rzAp+ruv_rzAn+ruv_rzB+uv_C+z.*ruv_rzC);

% Calculate the strain tensor components
Exx = rue_rx;
Eyy = run_ry;
Ezz = ruv_rz;
Exy = (rue_ry+run_rx)/2;
Exz = (rue_rz+ruv_rx)/2;
Eyz = (run_rz+ruv_ry)/2;

% Transform the strain tensor components from the Okada (1992) coordinate 
% system into EFCS
[Exx,Eyy,Ezz,Exy,Exz,Eyz] = TensTrans(Exx,Eyy,Ezz,Exy,Exz,Eyz,Rz');

% Calculate the stress tensor components
Sxx = 2*mu*Exx+lambda*(Exx+Eyy+Ezz);
Syy = 2*mu*Eyy+lambda*(Exx+Eyy+Ezz);
Szz = 2*mu*Ezz+lambda*(Exx+Eyy+Ezz);
Sxy = 2*mu*Exy;
Sxz = 2*mu*Exz;
Syz = 2*mu*Eyz;

Strain = [Exx,Eyy,Ezz,Exy,Exz,Eyz];
Stress = [Sxx,Syy,Szz,Sxy,Sxz,Syz];


function [Txx2,Tyy2,Tzz2,Txy2,Txz2,Tyz2]=TensTrans(Txx1,Tyy1,Tzz1,Txy1,...
    Txz1,Tyz1,A)
% TensTrans transforms the coordinates of tensors,from x1y1z1 coordinate
% system to x2y2z2 coordinate system. "A" is the transformation matrix,
% whose columns e1,e2 and e3 are the unit base vectors of the x1y1z1. The
% coordinates of e1,e2 and e3 in A must be given in x2y2z2. The transpose
% of A (i.e., A') does the transformation from x2y2z2 into x1y1z1.

Txx2 = A(1)^2*Txx1+2*A(1)*A(4)*Txy1+2*A(1)*A(7)*Txz1+2*A(4)*A(7)*Tyz1+...
    A(4)^2*Tyy1+A(7)^2*Tzz1;
Tyy2 = A(2)^2*Txx1+2*A(2)*A(5)*Txy1+2*A(2)*A(8)*Txz1+2*A(5)*A(8)*Tyz1+...
    A(5)^2*Tyy1+A(8)^2*Tzz1;
Tzz2 = A(3)^2*Txx1+2*A(3)*A(6)*Txy1+2*A(3)*A(9)*Txz1+2*A(6)*A(9)*Tyz1+...
    A(6)^2*Tyy1+A(9)^2*Tzz1;
Txy2 = A(1)*A(2)*Txx1+(A(1)*A(5)+A(2)*A(4))*Txy1+(A(1)*A(8)+...
    A(2)*A(7))*Txz1+(A(8)*A(4)+A(7)*A(5))*Tyz1+A(5)*A(4)*Tyy1+...
    A(7)*A(8)*Tzz1;
Txz2 = A(1)*A(3)*Txx1+(A(1)*A(6)+A(3)*A(4))*Txy1+(A(1)*A(9)+...
    A(3)*A(7))*Txz1+(A(9)*A(4)+A(7)*A(6))*Tyz1+A(6)*A(4)*Tyy1+...
    A(7)*A(9)*Tzz1;
Tyz2 = A(2)*A(3)*Txx1+(A(3)*A(5)+A(2)*A(6))*Txy1+(A(3)*A(8)+...
    A(2)*A(9))*Txz1+(A(8)*A(6)+A(9)*A(5))*Tyz1+A(5)*A(6)*Tyy1+...
    A(8)*A(9)*Tzz1;


function [ue,un,uv]=uC(x,y,z,d,depth,dip,r,q,s,t,alpha)
A3 = 1-3*x.^2./r.^2;
ue = -(1-alpha)*(3*x.*s)./r.^5+alpha*15*depth*x.*q.^2./r.^7-...
    alpha*3*x.*z./r.^5;
un = (1-alpha)*(sind(2*dip)-3*y.*s./r.^2)./r.^3+alpha*3*depth./r.^5.*...
    (t-y+5*y.*q.^2./r.^2)-alpha*3*y.*z./r.^5;
uv = -(1-alpha)*(1-A3*sind(dip)^2)./r.^3-alpha*3*depth./r.^5.*...
    (s-d+5*d.*q.^2./r.^2)+alpha*3*d.*z./r.^5;

% Calculate partial derivatives with respect to X
function [rue_rx,run_rx,ruv_rx]=ruA_rx(x,y,d,r,q,s,t,alpha)
A3 = 1-3*x.^2./r.^2;
A5 = 1-5*x.^2./r.^2;
rue_rx = (1-alpha)/2*A3./r.^3-alpha/2*3*q.^2./r.^5.*A5;
run_rx = -(1-alpha)/2*3*x.*t./r.^5+alpha/2*15*x.*y.*q.^2./r.^7;
ruv_rx = -(1-alpha)/2*3*x.*s./r.^5+alpha/2*15*x.*d.*q.^2./r.^7;

function [rue_rx,run_rx,ruv_rx]=ruB_rx(x,y,d,depth,dip,r,q,alpha)
A3 = 1-3*x.^2./r.^2;
A5 = 1-5*x.^2./r.^2;
J1 = -3*x.*y.*((3*r+d)./r.^3./(r+d).^3-x.^2.*(5*r.^2+4*r.*d+d.^2)./...
    r.^5./(r+d).^4);
J2 = 1./r.^3-3./r./(r+d).^2+3*x.^2.*y.^2.*(5*r.^2+4*r.*d+d.^2)./r.^5./...
    (r+d).^4;
J3 = A3./r.^3-J2;
K2 = -x.*((2*r+d)./r.^3./(r+d).^2-y.^2.*(8*r.^2+9*r.*d+3*d.^2)./r.^5./...
    (r+d).^3);
K3 = -3*x.*d./r.^5-K2;
rue_rx = 3*q.^2./r.^5.*A5-(1-alpha)/alpha*J3*sind(dip)^2;
run_rx = -15*x.*y.*q.^2./r.^7-(1-alpha)/alpha*J1*sind(dip)^2;
ruv_rx = -15*depth.*x.*q.^2./r.^7-(1-alpha)/alpha*K3*sind(dip)^2;

function [rue_rx,run_rx,ruv_rx]=ruC_rx(x,y,z,d,depth,dip,r,q,s,t,alpha)
A5 = 1-5*x.^2./r.^2;
A7 = 1-7*x.^2./r.^2;
rue_rx = -(1-alpha)*3*s./r.^5.*A5+alpha*15*depth.*q.^2./r.^7.*A7-...
    alpha*3*z./r.^5.*A5;
run_rx = -(1-alpha)*3*x./r.^5.*(sind(2*dip)-5*y.*s./r.^2)-...
    alpha*15*depth.*x./r.^7.*...
    (t-y+7*y.*q.^2./r.^2)+alpha*15*x.*y.*z./r.^7;
ruv_rx = (1-alpha)*3*x./r.^5.*(1-(2+A5)*sind(dip)^2)+alpha*15*depth*...
    x./r.^7.*(s-d+7*d.*q.^2./r.^2)-alpha*15*x.*d.*z./r.^7;


% Calculate partial derivatives with respect to Y
function [rue_ry,run_ry,ruv_ry]=ruA_ry(x,y,d,dip,r,q,s,t,alpha,W)
rue_ry = -(1-alpha)/2*3*x.*y./r.^5-alpha/2*3*x.*q./r.^5.*W;
run_ry = (1-alpha)/2./r.^3.*(cosd(2*dip)-3*y.*t./r.^2)-alpha/2*3*y.*...
    q./r.^5.*W-alpha/2*3*q.^2./r.^5;
ruv_ry = (1-alpha)/2./r.^3.*(sind(2*dip)-3*y.*s./r.^2)-alpha/2*3*d.*...
    q./r.^5.*W;

function [rue_ry,run_ry,ruv_ry]=ruB_ry(x,y,d,depth,dip,r,q,alpha,W)
J1 = -3*x.*y.*((3*r+d)./r.^3./(r+d).^3-x.^2.*(5*r.^2+4*r.*d+d.^2)./...
    r.^5./(r+d).^4);
J2 = 1./r.^3-3./r./(r+d).^2+3*x.^2.*y.^2.*(5*r.^2+4*r.*d+d.^2)./r.^5./...
    (r+d).^4;
K1 = -y.*((2*r+d)./r.^3./(r+d).^2-x.^2.*(8*r.^2+9*r.*d+3*d.^2)./r.^5./...
    (r+d).^3);
rue_ry = 3*x.*q./r.^5.*W-(1-alpha)/alpha*J1*sind(dip)^2;
run_ry = 3*y.*q./r.^5.*W+3*q.^2./r.^5-(1-alpha)/alpha*J2*sind(dip)^2;
ruv_ry = 3*depth.*q./r.^5.*W-(1-alpha)/alpha*K1*sind(dip)^2;

function [rue_ry,run_ry,ruv_ry]=ruC_ry(x,y,z,d,depth,dip,r,q,s,t,alpha)
A5 = 1-5*x.^2./r.^2;
B5 = 1-5*y.^2./r.^2;
B7 = 1-7*y.^2./r.^2;
rue_ry = -(1-alpha)*3*x./r.^5.*(sind(2*dip)-5*y.*s./r.^2)-alpha*15*...
    depth.*x./r.^7.*(t-y+7*y.*q.^2./r.^2)+alpha*15*x.*y.*z./r.^7;
run_ry = -(1-alpha)*3./r.^5.*(2*y*sind(2*dip)+s.*B5)-alpha*3*depth./...
    r.^5.*(2*sind(dip)^2+10*y.*(t-y)./r.^2-5*q.^2./r.^2.*B7)-alpha*3*z./...
    r.^5.*B5;
ruv_ry = (1-alpha)*3*y./r.^5.*(1-A5*sind(dip)^2)+alpha*3*depth./r.^5.*...
    ((3+A5).*sind(2*dip)-5*y.*d./r.^2.*(2-7*q.^2./r.^2))-alpha*15*y.*d.*...
    z./r.^7;


% Calculate partial derivatives with respect to Z
function [rue_rz,run_rz,ruv_rz]=ruA_rz(x,y,d,dip,r,q,s,t,alpha,Wi)
rue_rz = (1-alpha)/2*3*x.*d./r.^5-alpha/2*3*x.*q./r.^5.*Wi;
run_rz = -(1-alpha)/2./r.^3.*(sind(2*dip)-3*d.*t./r.^2)-alpha/2*3*y.*...
    q./r.^5.*Wi;
ruv_rz = (1-alpha)/2./r.^3.*(cosd(2*dip)+3*d.*s./r.^2)-alpha/2*3*d.*...
    q./r.^5.*Wi+alpha/2*3*q.^2./r.^5;

function [rue_rz,run_rz,ruv_rz]=ruB_rz(x,y,d,depth,dip,r,q,alpha,Wi)
A3 = 1-3*x.^2./r.^2;
K1 = -y.*((2*r+d)./r.^3./(r+d).^2-x.^2.*(8*r.^2+9*r.*d+3*d.^2)./r.^5./...
    (r+d).^3);
K2 = -x.*((2*r+d)./r.^3./(r+d).^2-y.^2.*(8*r.^2+9*r.*d+3*d.^2)./r.^5./...
    (r+d).^3);
K3 = -3*x.*d./r.^5-K2;
rue_rz = 3*x.*q./r.^5.*Wi+(1-alpha)/alpha*K3*sind(dip)^2;
run_rz = 3*y.*q./r.^5.*Wi+(1-alpha)/alpha*K1*sind(dip)^2;
ruv_rz = 3*depth.*q./r.^5.*Wi-(1-alpha)/alpha*A3./r.^3*sind(dip)^2;

function [rue_rz,run_rz,ruv_rz]=ruC_rz(x,y,z,d,depth,dip,r,q,s,alpha)
A5 = 1-5*x.^2./r.^2;
B5 = 1-5*y.^2./r.^2;
C5 = 1-5*d.^2./r.^2;
C7 = 1-7*d.^2./r.^2;
rue_rz = -(1-alpha)*3*x./r.^5.*(cosd(2*dip)+5*d.*s./r.^2)+alpha*15*...
    depth*x./r.^7.*(s-d+7*d.*q.^2./r.^2)-alpha*3*x./r.^5.*(1+5*d.*z./r.^2);
run_rz = (1-alpha)*3./r.^5.*(d.*B5*sind(2*dip)-y.*C5*cosd(2*dip))+...
    alpha*3*depth./r.^5.*((3+A5)*sind(2*dip)-5*y.*d./r.^2.*...
    (2-7*q.^2./r.^2))-alpha*3*y./r.^5.*(1+5*d.*z./r.^2);
ruv_rz = -(1-alpha)*3*d./r.^5.*(1-A5*sind(dip)^2)-alpha*3*depth./r.^5.*...
    (cosd(2*dip)+10*d.*(s-d)./r.^2-5*q.^2./r.^2.*C7)-alpha*3*z./r.^5.*...
    (1+C5);
