function [Stress,Strain]=RDstressFS(X,Y,Z,X0,Y0,depth,L,W,plunge,dip,...
    strike,rake,slip,opening,mu,lambda,RefPoint)
% RDstressFS
% calculates stresses and strains associated with a rectangular dislocation
% in a uniform elastic full-space.
%
% RD: Rectangular Dislocation
% EFCS: Earth-Fixed Coordinate System
% RDCS: Rectangular Dislocation Coordinate System
% ADCS: Angular Dislocation Coordinate System
% (The origin of the RDCS is the RD centroid. The axes of the RDCS are 
% aligned with the strike, dip and normal vectors of the RD, respectively.)
%
% INPUTS
% X, Y and Z:
% Coordinates of calculation points in EFCS (East, North, Up). X, Y and Z
% must have the same size.
%
% X0, Y0 and depth:
% Horizontal coordinates (in EFCS) and depth of the "RD reference point".
% The depth must be a positive value. The RD reference point is specified
% by the "RefPoint" value (see below). X0, Y0 and depth have the same unit
% as X, Y and Z.
%
% L and W:
% The length and width of the RD (see the note on the RD geometry in the
% following). L and W have the same unit as X, Y and Z.
%
% plunge, dip and strike:
% Rotation angles that specify the orientation of the RD in space (see the
% note on the RD geometry in the following). The input values must be in
% degrees.
%
% rake:
% The angle between the slip direction on the RD, and the intersection of
% the extended RD plane and the XY plane (of the EFCS). The input value
% must be in degrees.
% Note that rake is completely independent of the plunge angle!
%
% slip:
% The RD slip value (norm of the Burgers vector projection onto the RD
% plane).
%
% opening:
% The RD opening (tensile component of the Burgers vector).
%
% mu and lambda:
% Lam constants.
%
% RefPoint:
% specifies the RD reference point. The RefPoint can take the following
% values:
% 1) 'Pc' is the RD centroid,
% 2) 'P1' is the first RD vertex,
% 3) 'P2' is the second RD vertex,
% 4) 'P3' is the third RD vertex,
% 5) 'P4' is the fourth RD vertex,
% 6) 'mP12' or 'mP21' is the midpoint of the P1P2,
% 7) 'mP23' or 'mP32' is the midpoint of the P2P3,
% 8) 'mP34' or 'mP43' is the midpoint of the P3P4,
% 9) 'mP14' or 'mP41' is the midpoint of the P1P4. See Figure 2 in the
% reference journal article (cited below) for the geometry of the RD and
% its vertices.
% The coordinates of the RD reference point in EFCS are (X0,Y0,-depth).
% (see the note on the RD geometry in the following).
%
%
%
% *** A note on the RD geometry ***
% To specify the orientation and location of any arbitrary RD in space we
% start with an initial RD that has two edges of length "W" along the X
% axis and has two edges of length "L" along the Y axis and its centroid is
% located on the origin of the EFCS, respectively. We apply the following
% rotations to this initial RD:
% 1) a "clockwise" rotation about the Z axis by an angle "plunge",
% 2) an "anticlockwise" rotation about the Y axis by an angle "dip",
% 3) a "clockwise" rotation about the Z axis by an angle "strike".
% This fixes the orientation of the RD in space. We then fix the location
% of the RD by applying a translation that places the RD reference point at
% the point (X0,Y0,-depth).
%
%
%
% OUTPUTS
% Stress:
% Calculated stress tensor components in EFCS. The six columns of Stress
% are Sxx, Syy, Szz, Sxy, Sxz and Syz, respectively. The stress components
% have the same unit as Lam constants.
%
% Strain:
% Calculated strain tensor components in EFCS. The six columns of Strain
% are Exx, Eyy, Ezz, Exy, Exz and Eyz, respectively. The strain components
% are dimensionless.
%
%
% Example: Calculate and plot the first component of stress tensor on a
% regular grid.
%
% [X,Y,Z] = meshgrid(-10:.05:10,-8:.05:8,4);
% X0 = 3; Y0 = 1; depth = 0; L = 2; W = 1; plunge = 10; dip = 75;
% strike = 30; rake = 20; slip = 1; opening = 0.15; mu = 0.33e11;
% lambda = 0.33e11; RefPoint = 'Pc';
% [Stress,Strain] = RDstressFS(X,Y,Z,X0,Y0,depth,L,W,plunge,dip,strike,...
%     rake,slip,opening,mu,lambda,RefPoint);
% figure
% surf(X,Y,reshape(Stress(:,1),size(X)),'edgecolor','none')
% view(2)
% axis equal
% axis tight
% set(gcf,'renderer','painters')

% Reference journal articles:
% 1)
% Nikkhoo, M., Walter, T. R., Lundgren, P. R., Prats-Iraola, P. (2017):
% Compound dislocation models (CDMs) for volcano deformation analyses.
% Submitted to Geophysical Journal International, doi: 10.1093/gji/ggw427
% 
% 2)
% Yoffe, E. H. (1960):
% The angular dislocation, Philos. Mag., 5(50), 161175,
% doi: 10.1080/14786436008243299

% Copyright (c) 2016 Mehdi Nikkhoo
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files
% (the "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.

% I appreciate any comments or bug reports.

% Mehdi Nikkhoo
% created: 2013.1.25
% Last modified: 2022.10.24
%
% Section 2.1, Physics of Earthquakes and Volcanoes
% Department 2, Geophysics
% Helmholtz Centre Potsdam
% German Research Centre for Geosciences (GFZ)
%
% email:
% mehdi.nikkhoo@gfz-potsdam.de
% mehdi.nikkhoo@gmail.com
%
% website:
% http://www.volcanodeformation.com

nu = lambda/(lambda+mu)/2; % Poisson's ratio

bx = opening; % Tensile-slip
by = slip.*cosd(rake); % Strike-slip
bz = slip.*sind(rake); % Dip-slip

X = X(:);
Y = Y(:);
Z = Z(:);

Rz1 = [cosd(plunge) sind(plunge) 0;-sind(plunge) cosd(plunge) 0;0 0 1];
Ry = [cosd(dip) 0 sind(dip);0 1 0;-sind(dip) 0 cosd(dip)];
Rz2 = [cosd(strike) sind(strike) 0;-sind(strike) cosd(strike) 0;0 0 1];
Rt = Rz2*Ry*Rz1;

Pt1 = [-W/2 L/2 0]';
Pt2 = [-W/2 -L/2 0]';
Pt3 = [W/2 -L/2 0]';
Pt4 = [W/2 L/2 0]';

if strcmpi(RefPoint,'Pc')
    Ptr = [0,0,0]';
elseif strcmpi(RefPoint,'P1')
    Ptr = [-W/2 L/2 0]';
elseif strcmpi(RefPoint,'P2')
    Ptr = [-W/2 -L/2 0]';
elseif strcmpi(RefPoint,'P3')
    Ptr = [W/2 -L/2 0]';
elseif strcmpi(RefPoint,'P4')
    Ptr = [W/2 L/2 0]';
elseif strcmpi(RefPoint,'mP12') || strcmpi(RefPoint,'mP21')
    Ptr = [-W/2 0 0]';
elseif strcmpi(RefPoint,'mP23') || strcmpi(RefPoint,'mP32')
    Ptr = [0 -L/2 0]';
elseif strcmpi(RefPoint,'mP34') || strcmpi(RefPoint,'mP43')
    Ptr = [W/2 0 0]';
elseif strcmpi(RefPoint,'mP14') || strcmpi(RefPoint,'mP41')
    Ptr = [0 L/2 0]';
else
    error('Undefined RD reference point!')
end

Pr = [X0,Y0,-depth]'-Rt*Ptr;
P1 = Rt*Pt1+Pr;
P2 = Rt*Pt2+Pr;
P3 = Rt*Pt3+Pr;
P4 = Rt*Pt4+Pr;

% Calculate unit strike, dip and normal to the RD vectors:
eZ = [0 0 1]';
Vnorm = Rt*eZ;
Vstrike = [sind(strike) cosd(strike) 0]';
Vdip = cross(Vnorm,Vstrike);

Pm = (P1+P2+P3+P4)/4; % RD centroid

% Transform coordinates and slip vector components from EFCS into RDCS
p1 = zeros(3,1);
p2 = zeros(3,1);
p3 = zeros(3,1);
p4 = zeros(3,1);

At = [Vnorm Vstrike Vdip]';
[x,y,z] = CoordTrans(X'-Pm(1),Y'-Pm(2),Z'-Pm(3),At);
[p1(1),p1(2),p1(3)] = CoordTrans(P1(1)-Pm(1),P1(2)-Pm(2),P1(3)-Pm(3),At);
[p2(1),p2(2),p2(3)] = CoordTrans(P2(1)-Pm(1),P2(2)-Pm(2),P2(3)-Pm(3),At);
[p3(1),p3(2),p3(3)] = CoordTrans(P3(1)-Pm(1),P3(2)-Pm(2),P3(3)-Pm(3),At);
[p4(1),p4(2),p4(3)] = CoordTrans(P4(1)-Pm(1),P4(2)-Pm(2),P4(3)-Pm(3),At);

% Calculate the unit vectors along RD sides in RDCS
e12 = (p2-p1)/norm(p2-p1);
e23 = (p3-p2)/norm(p3-p2);
e34 = (p4-p3)/norm(p4-p3);
e14 = (p4-p1)/norm(p4-p1);

% Determine the best arteact-free configuration for each calculation point
Rectmode = rectmodefinder(y,z,x,p1(2:3),p2(2:3),p3(2:3),p4(2:3));
casepLog = Rectmode==1;
casenLog = Rectmode==-1;
casezLog = Rectmode==0;
xp = x(casepLog);
yp = y(casepLog);
zp = z(casepLog);
xn = x(casenLog);
yn = y(casenLog);
zn = z(casenLog);

% Configuration I
if nnz(casepLog)~=0
    % Calculate first angular dislocation contribution
    [Exx1Tp,Eyy1Tp,Ezz1Tp,Exy1Tp,Exz1Tp,Eyz1Tp] = RDSetupS(xp,yp,zp,...
        bx,by,bz,nu,p1,e14);
    % Calculate second angular dislocation contribution
    [Exx2Tp,Eyy2Tp,Ezz2Tp,Exy2Tp,Exz2Tp,Eyz2Tp] = RDSetupS(xp,yp,zp,...
        bx,by,bz,nu,p2,-e12);
    % Calculate third angular dislocation contribution
    [Exx3Tp,Eyy3Tp,Ezz3Tp,Exy3Tp,Exz3Tp,Eyz3Tp] = RDSetupS(xp,yp,zp,...
        bx,by,bz,nu,p3,-e23);
    % Calculate fourth angular dislocation contribution
    [Exx4Tp,Eyy4Tp,Ezz4Tp,Exy4Tp,Exz4Tp,Eyz4Tp] = RDSetupS(xp,yp,zp,...
        bx,by,bz,nu,p4,-e34);
end

% Configuration II
if nnz(casenLog)~=0
    % Calculate first angular dislocation contribution
    [Exx1Tn,Eyy1Tn,Ezz1Tn,Exy1Tn,Exz1Tn,Eyz1Tn] = RDSetupS(xn,yn,zn,...
        bx,by,bz,nu,p1,-e14);
    % Calculate second angular dislocation contribution
    [Exx2Tn,Eyy2Tn,Ezz2Tn,Exy2Tn,Exz2Tn,Eyz2Tn] = RDSetupS(xn,yn,zn,...
        bx,by,bz,nu,p2,e12);
    % Calculate third angular dislocation contribution
    [Exx3Tn,Eyy3Tn,Ezz3Tn,Exy3Tn,Exz3Tn,Eyz3Tn] = RDSetupS(xn,yn,zn,...
        bx,by,bz,nu,p3,e23);
    % Calculate fourth angular dislocation contribution
    [Exx4Tn,Eyy4Tn,Ezz4Tn,Exy4Tn,Exz4Tn,Eyz4Tn] = RDSetupS(xn,yn,zn,...
        bx,by,bz,nu,p4,e34);
end

% Calculate the strain tensor components in RDCS
if nnz(casepLog)~=0
    exx(casepLog,1) = Exx1Tp+Exx2Tp+Exx3Tp+Exx4Tp;
    eyy(casepLog,1) = Eyy1Tp+Eyy2Tp+Eyy3Tp+Eyy4Tp;
    ezz(casepLog,1) = Ezz1Tp+Ezz2Tp+Ezz3Tp+Ezz4Tp;
    exy(casepLog,1) = Exy1Tp+Exy2Tp+Exy3Tp+Exy4Tp;
    exz(casepLog,1) = Exz1Tp+Exz2Tp+Exz3Tp+Exz4Tp;
    eyz(casepLog,1) = Eyz1Tp+Eyz2Tp+Eyz3Tp+Eyz4Tp;
end
if nnz(casenLog)~=0
    exx(casenLog,1) = Exx1Tn+Exx2Tn+Exx3Tn+Exx4Tn;
    eyy(casenLog,1) = Eyy1Tn+Eyy2Tn+Eyy3Tn+Eyy4Tn;
    ezz(casenLog,1) = Ezz1Tn+Ezz2Tn+Ezz3Tn+Ezz4Tn;
    exy(casenLog,1) = Exy1Tn+Exy2Tn+Exy3Tn+Exy4Tn;
    exz(casenLog,1) = Exz1Tn+Exz2Tn+Exz3Tn+Exz4Tn;
    eyz(casenLog,1) = Eyz1Tn+Eyz2Tn+Eyz3Tn+Eyz4Tn;
end
if nnz(casezLog)~=0
    exx(casezLog,1) = nan;
    eyy(casezLog,1) = nan;
    ezz(casezLog,1) = nan;
    exy(casezLog,1) = nan;
    exz(casezLog,1) = nan;
    eyz(casezLog,1) = nan;
end

% Transform the strain tensor components from RDCS into EFCS
[Exx,Eyy,Ezz,Exy,Exz,Eyz] = TensTrans(exx,eyy,ezz,exy,exz,eyz,...
    [Vnorm,Vstrike,Vdip]);

% Calculate the stress tensor components in EFCS
Sxx = 2*mu*Exx+lambda*(Exx+Eyy+Ezz);
Syy = 2*mu*Eyy+lambda*(Exx+Eyy+Ezz);
Szz = 2*mu*Ezz+lambda*(Exx+Eyy+Ezz);
Sxy = 2*mu*Exy;
Sxz = 2*mu*Exz;
Syz = 2*mu*Eyz;

Strain = [Exx,Eyy,Ezz,Exy,Exz,Eyz];
Stress = [Sxx,Syy,Szz,Sxy,Sxz,Syz];

function [Txx2,Tyy2,Tzz2,Txy2,Txz2,Tyz2]=TensTrans(Txx1,Tyy1,Tzz1,Txy1,...
    Txz1,Tyz1,A)
% TensTrans Transforms the coordinates of tensors,from x1y1z1 coordinate
% system to x2y2z2 coordinate system. "A" is the transformation matrix,
% whose columns e1,e2 and e3 are the unit base vectors of the x1y1z1. The
% coordinates of e1,e2 and e3 in A must be given in x2y2z2. The transpose
% of A (i.e., A') does the transformation from x2y2z2 into x1y1z1.

Txx2 = A(1)^2*Txx1+2*A(1)*A(4)*Txy1+2*A(1)*A(7)*Txz1+2*A(4)*A(7)*Tyz1+...
    A(4)^2*Tyy1+A(7)^2*Tzz1;
Tyy2 = A(2)^2*Txx1+2*A(2)*A(5)*Txy1+2*A(2)*A(8)*Txz1+2*A(5)*A(8)*Tyz1+...
    A(5)^2*Tyy1+A(8)^2*Tzz1;
Tzz2 = A(3)^2*Txx1+2*A(3)*A(6)*Txy1+2*A(3)*A(9)*Txz1+2*A(6)*A(9)*Tyz1+...
    A(6)^2*Tyy1+A(9)^2*Tzz1;
Txy2 = A(1)*A(2)*Txx1+(A(1)*A(5)+A(2)*A(4))*Txy1+(A(1)*A(8)+...
    A(2)*A(7))*Txz1+(A(8)*A(4)+A(7)*A(5))*Tyz1+A(5)*A(4)*Tyy1+...
    A(7)*A(8)*Tzz1;
Txz2 = A(1)*A(3)*Txx1+(A(1)*A(6)+A(3)*A(4))*Txy1+(A(1)*A(9)+...
    A(3)*A(7))*Txz1+(A(9)*A(4)+A(7)*A(6))*Tyz1+A(6)*A(4)*Tyy1+...
    A(7)*A(9)*Tzz1;
Tyz2 = A(2)*A(3)*Txx1+(A(3)*A(5)+A(2)*A(6))*Txy1+(A(3)*A(8)+...
    A(2)*A(9))*Txz1+(A(8)*A(6)+A(9)*A(5))*Tyz1+A(5)*A(6)*Tyy1+...
    A(8)*A(9)*Tzz1;

function [X1,X2,X3]=CoordTrans(x1,x2,x3,A)
% CoordTrans transforms the coordinates of the vectors, from
% x1x2x3 coordinate system to X1X2X3 coordinate system. "A" is the
% transformation matrix, whose columns e1,e2 and e3 are the unit base
% vectors of the x1x2x3. The coordinates of e1,e2 and e3 in A must be given
% in X1X2X3. The transpose of A (i.e., A') will transform the coordinates
% from X1X2X3 into x1x2x3.

x1 = x1(:);
x2 = x2(:);
x3 = x3(:);
r = A*[x1';x2';x3'];
X1 = r(1,:)';
X2 = r(2,:)';
X3 = r(3,:)';

function [rectmode]=rectmodefinder(x,y,z,p1,p2,p3,p4)
% rectmodefinder specifies the appropriate artefact-free configuration of
% the angular dislocations for the calculations. The input matrices x, y
% and z share the same size and correspond to the y, z and x coordinates in
% the RDCS, respectively. p1, p2, p3 and p4 are two-component matrices that
% represent the y and z coordinates of the RD vertices in the RDCS,
% respectively. The components of the output (rectmode) corresponding to
% each calculation points, are 1 for the first configuration, -1 for the
% second configuration and 0 for the calculation point that lie on the RD
% edges.

x = x(:);
y = y(:);
z = z(:);

p1 = p1(:);
p2 = p2(:);
p3 = p3(:);
p4 = p4(:);

pm = (p1+p2+p3+p4)/4; % centroid position vector
r21 = p1-p2;
e21 = r21/sqrt(r21'*r21); % unit vector along new x-axis
r41 = p1-p4;
e41 = r41/sqrt(r41'*r41); % unit vector along new y-axis

% Transform the coordinates to the new coordinate system
x = x-pm(1);
y = y-pm(2);

A = [e21 e41];
r = A'*[x';y'];
% new coordinates of the calculation points
x = r(1,:)';
y = r(2,:)';

r = A'*[p1-pm,p2-pm,p3-pm,p4-pm];
% new coordinates of the rectangle vertices
P1 = r(:,1);
P2 = r(:,2);
P3 = r(:,3);
P4 = r(:,4);

rectmode = ones(length(x),1);
% Partition the RD plane using the bisectors of the RD angles and RD sides
rectmode(x>=0 & y>=0 & ((y-P1(2))<(x-P1(1)))) = -1;
rectmode(x<=0 & y>=0 & ((y-P2(2))>-(x-P2(1)))) = -1;
rectmode(x<=0 & y<=0 & ((y-P3(2))>(x-P3(1)))) = -1;
rectmode(x>=0 & y<=0 & ((y-P4(2))<-(x-P4(1)))) = -1;
rectmode(x<P1(1) & x>P3(1) & y<P1(2) & y>P3(2)) = -1;
rectmode((x==P1(1) | x==P3(1)) & y<=P1(2) & y>=P3(2) & z==0) = 0;
rectmode((y==P1(2) | y==P3(2)) & x<=P1(1) & x>=P3(1) & z==0) = 0;

function [exx,eyy,ezz,exy,exz,eyz]=RDSetupS(x,y,z,bx,by,bz,nu,RDVertex,...
    SideVec)
% RDSetupS transforms coordinates of the calculation points and slip vector
% components from ADCS into RDCS. It then calculates the strains in ADCS
% and transforms them into RDCS.

% Transformation matrix
A = [[SideVec(3);-SideVec(2)] SideVec(2:3)]';

% Transform coordinates of the calculation points from RDCS into ADCS
r1 = A*[y'-RDVertex(2);z'-RDVertex(3)];
y1 = r1(1,:)';
z1 = r1(2,:)';

% Transform the in-plane slip vector components from RDCS into ADCS
r2 = A*[by;bz];
by1 = r2(1,:)';
bz1 = r2(2,:)';

% Calculate strains associated with an angular dislocation in ADCS
[exx,eyy,ezz,exy,exz,eyz] = AngDisStrain(x,y1,z1,-pi/2,bx,by1,bz1,nu);

% Transform strains from ADCS into RDCS
B = [[1 0 0];[zeros(2,1),A']]; % 3x3 Transformation matrix
[exx,eyy,ezz,exy,exz,eyz] = TensTrans(exx,eyy,ezz,exy,exz,eyz,B);

function [Exx,Eyy,Ezz,Exy,Exz,Eyz]=AngDisStrain(x,y,z,alpha,bx,by,bz,nu)
% AngDisStrain calculates the strains associated with an angular
% dislocation in an elastic full-space.

sinA = sin(alpha);
cosA = cos(alpha);
eta = y.*cosA-z.*sinA;
zeta = y.*sinA+z.*cosA;

x2 = x.^2;
y2 = y.^2;
z2 = z.^2;
r2 = x2+y2+z2;
r = sqrt(r2);
r3 = r.*r2;
rz = r.*(r-z);
r2z2 = r2.*(r-z).^2;
r3z = r3.*(r-z);

W = zeta-r;
W2 = W.^2;
Wr = W.*r;
W2r = W2.*r;
Wr3 = W.*r3;
W2r2 = W2.*r2;

C = (r*cosA-z)./Wr;
S = (r*sinA-y)./Wr;

% Partial derivatives of the Burgers' function
rFi_rx = (eta./r./(r-zeta)-y./r./(r-z))/4/pi;
rFi_ry = (x./r./(r-z)-cosA*x./r./(r-zeta))/4/pi;
rFi_rz = (sinA*x./r./(r-zeta))/4/pi;

Exx = bx.*(rFi_rx)+...
    bx/8/pi/(1-nu)*(eta./Wr+eta.*x2./W2r2-eta.*x2./Wr3+y./rz-...
    x2.*y./r2z2-x2.*y./r3z)-...
    by*x/8/pi/(1-nu).*(((2*nu+1)./Wr+x2./W2r2-x2./Wr3)*cosA+...
    (2*nu+1)./rz-x2./r2z2-x2./r3z)+...
    bz*x*sinA/8/pi/(1-nu).*((2*nu+1)./Wr+x2./W2r2-x2./Wr3);

Eyy = by.*(rFi_ry)+...
    bx/8/pi/(1-nu)*((1./Wr+S.^2-y2./Wr3).*eta+(2*nu+1)*y./rz-y.^3./r2z2-...
    y.^3./r3z-2*nu*cosA*S)-...
    by*x/8/pi/(1-nu).*(1./rz-y2./r2z2-y2./r3z+...
    (1./Wr+S.^2-y2./Wr3)*cosA)+...
    bz*x*sinA/8/pi/(1-nu).*(1./Wr+S.^2-y2./Wr3);

Ezz = bz.*(rFi_rz)+...
    bx/8/pi/(1-nu)*(eta./W./r+eta.*C.^2-eta.*z2./Wr3+y.*z./r3+...
    2*nu*sinA*C)-...
    by*x/8/pi/(1-nu).*((1./Wr+C.^2-z2./Wr3)*cosA+z./r3)+...
    bz*x*sinA/8/pi/(1-nu).*(1./Wr+C.^2-z2./Wr3);

Exy = bx.*(rFi_ry)./2+by.*(rFi_rx)./2-...
    bx/8/pi/(1-nu).*(x.*y2./r2z2-nu*x./rz+x.*y2./r3z-nu*x*cosA./Wr+...
    eta.*x.*S./Wr+eta.*x.*y./Wr3)+...
    by/8/pi/(1-nu)*(x2.*y./r2z2-nu*y./rz+x2.*y./r3z+nu*cosA*S+...
    x2.*y*cosA./Wr3+x2*cosA.*S./Wr)-...
    bz*sinA/8/pi/(1-nu).*(nu*S+x2.*S./Wr+x2.*y./Wr3);

Exz = bx.*(rFi_rz)./2+bz.*(rFi_rx)./2-...
    bx/8/pi/(1-nu)*(-x.*y./r3+nu*x*sinA./Wr+eta.*x.*C./Wr+...
    eta.*x.*z./Wr3)+...
    by/8/pi/(1-nu)*(-x2./r3+nu./r+nu*cosA*C+x2.*z*cosA./Wr3+...
    x2*cosA.*C./Wr)-...
    bz*sinA/8/pi/(1-nu).*(nu*C+x2.*C./Wr+x2.*z./Wr3);

Eyz = by.*(rFi_rz)./2+bz.*(rFi_ry)./2+...
    bx/8/pi/(1-nu).*(y2./r3-nu./r-nu*cosA*C+nu*sinA*S+eta*sinA*cosA./W2-...
    eta.*(y*cosA+z*sinA)./W2r+eta.*y.*z./W2r2-eta.*y.*z./Wr3)-...
    by*x/8/pi/(1-nu).*(y./r3+sinA*cosA^2./W2-cosA*(y*cosA+z*sinA)./...
    W2r+y.*z*cosA./W2r2-y.*z*cosA./Wr3)-...
    bz*x*sinA/8/pi/(1-nu).*(y.*z./Wr3-sinA*cosA./W2+(y*cosA+z*sinA)./...
    W2r-y.*z./W2r2);
